#!/usr/bin/env python3
# Hexcone clour space interpolate by Mike Cook

import time, random
from apa102bang import Apa102bang

def main():
  print("Hexcone colour space interpolate")
  init()
  while 1:
     genCols()
     showBetween()
     time.sleep(1.5)
     strip.setAll((0,0,0)) # blank
     strip.show()
     time.sleep(0.5)
     
def showBetween():
   print()
   print("HSV space from",c1,"to",c2)
   print("That is rgb ",hexToRGB(c1),hexToRGB(c2))
   strip.setLed(0,hexToRGB(c1))
   strip.setLed(7,hexToRGB(c2))
   strip.show()
   time.sleep(0.5)

   hInc = float(c2[0] - c1[0])/6.0
   sInc = float(c2[1] - c1[1])/6.0
   vInc = float(c2[2] - c1[2])/6.0
   h = c1[0] ; s = c1[1] ; v = c1[2]
   for p in range(1,7):
      h += hInc
      s += sInc
      v += vInc
      strip.setLed(p,hexToRGB((h,s,v)))
      print(hexToRGB((h,s,v)))
      strip.show()
      time.sleep(0.6)
  
def hexToRGB(col):
    h = col[0]
    s = col[1]    
    v = col[2]
    if s == 0.0 : # s is zero on the grey line
      v = int(col[2] * 255.0)
      return (v,v,v)
    # get h into range 0 to 360
    if h>=360.0:
      while h >= 360.0 :
        h -= 360.0
    if h < 0.0:
      while h < 0.0 :
        h += 360.0
    h = h/60.0
    i = int(h)
    f = h - i # fractional part of h
    p = int(v*(1.0-s) * 255)
    q = int(v*(1.0-(s*f)) * 255)
    t = int(v*(1.0-(s*(1.0-f)))* 255)
    v = int(v * 255)
    if i == 0:
      return (v,t,p)
    if i == 1:
      return (q,v,p)
    if i == 2:
      return (p,v,t)
    if i == 3:
      return (p,q,v)
    if i == 4:
      return (t,p,v)
    if i == 5:
      return (v,p,q)
    
  
def genCols(): # Generate two different random colours
   global c1, c2
   while 1 :
     for c in range(0,2): 
        h = float(int(random.random()*360.0))
        s = (random.random()+1)/2.0
        v = (random.random()+1)/2.0
        if c == 0:
           c1 = (h,s,v)
        else:
           c2 = (h,s,v)
     if abs(c1[0]-c2[0]) > 60.0 and abs(c1[1]-c2[1]) > 0.3 and abs(c1[2]-c2[2]) > 0.3:
        break
 
def init():
   global strip
   random.seed()
   length = 8 ; brightness = 5
   dataPin = 23 ; clockPin = 24 # Blinkt wiring
   strip = Apa102bang(length,dataPin,clockPin,brightness)
        
# Main program logic:
if __name__ == '__main__':    
    main()
